/* Get specific attribute of abbreviation.
   Copyright (C) 2003 Red Hat, Inc.
   Written by Ulrich Drepper <drepper@redhat.com>, 2003.

   This program is Open Source software; you can redistribute it and/or
   modify it under the terms of the Open Software License version 1.0 as
   published by the Open Source Initiative.

   You should have received a copy of the Open Software License along
   with this program; if not, you may obtain a copy of the Open Software
   License version 1.0 from http://www.opensource.org/licenses/osl.php or
   by writing the Open Source Initiative c/o Lawrence Rosen, Esq.,
   3001 King Ranch Road, Ukiah, CA 95482.   */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <assert.h>
#include <dwarf.h>
#include "libdwP.h"


int
dwarf_getabbrevattr (abbrev, idx, codep, formp)
     Dwarf_Abbrev *abbrev;
     size_t idx;
     unsigned int *codep;
     unsigned int *formp;
{
  if (abbrev == NULL)
    return -1;

  size_t cnt = 0;
  unsigned char *attrp = abbrev->attrp;
  unsigned int code;
  unsigned int form;

  do
    {
      /* Attribute code and form are encoded as ULEB128 values.  */
      get_uleb128 (code, attrp);
      get_uleb128 (form, attrp);

      /* Ifboth values are zero the index is out of range.  */
      if (code == 0 && form == 0)
	return -1;
    }
  while (cnt++ < idx);

  /* Store the result if requested.  */
  if (codep != NULL)
    *codep = code;
  if (formp != NULL)
    *formp = form;

  return 0;
}
